import customtkinter as ctk
import tkinter as tk
from PIL import Image, ImageTk
import os

class PreviewWindow(ctk.CTkToplevel):
    def __init__(self, parent, content_type="readme", content=None, image_path=None):
        super().__init__(parent)
        
        
        self.title("Prévisualisation")
        self.geometry("800x600")
        
        
        self.result = False
        
        
        self.grid_columnconfigure(0, weight=1)
        self.grid_rowconfigure(0, weight=1)
        
        
        self.main_frame = ctk.CTkFrame(self)
        self.main_frame.grid(row=0, column=0, padx=20, pady=20, sticky="nsew")
        self.main_frame.grid_columnconfigure(0, weight=1)
        self.main_frame.grid_rowconfigure(0, weight=1)
        
        
        if content_type == "readme":
            self.preview_text = ctk.CTkTextbox(
                self.main_frame,
                wrap="word",
                font=("Courier New", 12)
            )
            self.preview_text.grid(row=0, column=0, padx=10, pady=10, sticky="nsew")
            self.preview_text.insert("1.0", content)
            self.preview_text.configure(state="disabled")
        else:  
            self.preview_frame = ctk.CTkFrame(self.main_frame)
            self.preview_frame.grid(row=0, column=0, padx=10, pady=10, sticky="nsew")
            
            if os.path.exists(image_path):
                
                image = Image.open(image_path)
                
                
                frame_width = 760  
                frame_height = 520  
                
                
                ratio = min(frame_width/image.width, frame_height/image.height)
                new_width = int(image.width * ratio)
                new_height = int(image.height * ratio)
                
                image = image.resize((new_width, new_height), Image.Resampling.LANCZOS)
                photo = ImageTk.PhotoImage(image)
                
                self.image_label = ctk.CTkLabel(
                    self.preview_frame,
                    text=""
                )
                self.image_label.grid(row=0, column=0, padx=10, pady=10)
                self.image_label.configure(image=photo)
                self.image_label.image = photo  
            else:
                self.error_label = ctk.CTkLabel(
                    self.preview_frame,
                    text="Erreur : Image non trouvée",
                    font=("Helvetica", 14)
                )
                self.error_label.grid(row=0, column=0, padx=10, pady=10)
        
        
        self.button_frame = ctk.CTkFrame(self.main_frame)
        self.button_frame.grid(row=1, column=0, padx=10, pady=10, sticky="ew")
        self.button_frame.grid_columnconfigure((0, 1), weight=1)
        
        
        self.validate_button = ctk.CTkButton(
            self.button_frame,
            text="Valider",
            command=self.validate,
            font=("Helvetica", 14)
        )
        self.validate_button.grid(row=0, column=0, padx=10, pady=5)
        
        self.cancel_button = ctk.CTkButton(
            self.button_frame,
            text="Annuler",
            command=self.cancel,
            font=("Helvetica", 14)
        )
        self.cancel_button.grid(row=0, column=1, padx=10, pady=5)
        
        
        self.transient(parent)
        self.grab_set()
        
    def validate(self):
        self.result = True
        self.destroy()
        
    def cancel(self):
        self.result = False
        self.destroy()
