import os
from plantuml import PlantUML
from plantuml import PlantUMLHTTPError
import tempfile

class UMLGenerator:
    def __init__(self):
        self.url = "http://www.plantuml.com/plantuml/img/"
        self.plantuml = PlantUML(url=self.url)
        
    def log_message(self, message):
        
        print(message)
        if "ERREUR" in message:
            print("=== Génération interrompue ===")
            
    def generate_class_diagram(self, classes, output_path):
        uml_content = "@startuml\n\n"
        
        
        uml_content += "hide circle\n"
        uml_content += "allowmixing\n"
        uml_content += "skinparam classAttributeIconSize 0\n"
        uml_content += "skinparam classBackgroundColor #ffffb9\n"
        uml_content += "skinparam classBorderColor #800000\n"
        uml_content += "skinparam classArrowColor #800000\n"
        uml_content += "skinparam classFontColor black\n"
        uml_content += "skinparam classFontName Tahoma\n\n"
        
        
        for class_info in classes:
            uml_content += f"class {class_info['name']} {{\n"
            
            
            for attr in class_info['attributes']:
                uml_content += f"    +{attr['name']}: {attr['type']}\n"
            
            
            for method in class_info['methods']:
                params = ", ".join([f"{p[0]} {p[1]}" if isinstance(p, tuple) else p for p in method['parameters']])
                uml_content += f"    +{method['name']}({params}): {method['return_type']}\n"
            
            uml_content += "}\n\n"
            
            
            if class_info['extends']:
                uml_content += f"{class_info['extends']} <|-- {class_info['name']}\n"
            
            
            for interface in class_info['implements']:
                uml_content += f"{interface} <|.. {class_info['name']}\n"
            
            
            if 'associations' in class_info:
                for assoc_class in class_info['associations']:
                    uml_content += f"{class_info['name']} --> {assoc_class}\n"  
            
            
            if 'dependencies' in class_info:
                for dep_class in class_info['dependencies']:
                    uml_content += f"{class_info['name']} ..> {dep_class}\n"  

            
            if 'aggregations' in class_info:
                for agg_class in class_info['aggregations']:
                    uml_content += f"{class_info['name']} o--> {agg_class}\n"  
            
            
            if 'compositions' in class_info:
                for comp_class in class_info['compositions']:
                    uml_content += f"{class_info['name']} *--> {comp_class}\n"  

        uml_content += "@enduml"

       

        output_puml_file = os.path.join(output_path, 'class_diagram.puml')

        with open(output_puml_file, 'w', encoding='utf-8') as f:
            f.write(uml_content)

    
        png_file = os.path.join(output_path, "class_diagram.png")
        self.plantuml.processes_file(output_puml_file, outfile=png_file)
        return png_file  

    def generate_documentation(self, classes, output_path):
        doc_content = "# Documentation du Projet\n\n"
        
        
        for class_info in classes:
            doc_content += f"## Classe {class_info['name']}\n\n"
            
            if class_info['extends']:
                doc_content += f"Hérite de: `{class_info['extends']}`\n\n"
            
            if class_info['implements']:
                doc_content += f"Implémente: {', '.join(class_info['implements'])}\n\n"
            
            doc_content += "### Attributs\n\n"
            for attr in class_info['attributes']:
                doc_content += f"- `{attr['name']}`: {attr['type']}\n"
            
            doc_content += "\n### Méthodes\n\n"
            for method in class_info['methods']:
                params = ", ".join([f"{p[0]} {p[1]}" if isinstance(p, tuple) else p for p in method['parameters']])
                doc_content += f"#### {method['name']}\n"
                doc_content += f"- Retour: `{method['return_type']}`\n"
                doc_content += f"- Paramètres: `{params}`\n\n"
            
            doc_content += "---\n\n"
        
        
        doc_file = os.path.join(output_path, "documentation.md")
        with open(doc_file, "w", encoding="utf-8") as f:
            f.write(doc_content)
        
        return doc_file
